import unittest

import bgpkit


class TestIntegration(unittest.TestCase):

    def test_parser(self):
        parser = bgpkit.Parser(url="https://spaces.bgpkit.org/parser/update-example",
                               filters={"peer_ips": "185.1.8.65, 2001:7f8:73:0:3:fa4:0:1"})
        elems = parser.parse_all()
        assert len(elems) == 4227

    def test_broker(self):
        broker = bgpkit.Broker()
        items = broker.query(start_ts=1643760000, end_ts=1643761200)
        for item in items:
            print(item)
        print(len(items))
        assert len(items) == 58

        # elems = bgpkit.Parser(items[0].url).parse_all()
        # assert len(elems) == 222467

    def test_roas(self):
        roas = bgpkit.Roas()
        data = roas.query(debug=True, asn=3333, date="2018-01-01")
        for entry in data:
            print(entry)
        assert len(data) == 10

        assert len(roas.query()) == 0



from dataclasses import dataclass

import requests as requests


def check_type(value: any, ty: type) -> bool:
    try:
        ty(value)
        return True
    except ValueError:
        raise ValueError("invalid option input")


@dataclass
class BrokerItem:
    collector_id: str
    data_type: str
    timestamp: int
    url: str


class Broker:

    def __init__(self, api_url: str = "https://api.broker.bgpkit.com/v1", page_size: int = 100):
        self.base_url = api_url.strip()
        self.page_size = int(page_size)

    def query(self,
              start_ts: int = None,
              end_ts: int = None,
              collector: str = None,
              project: str = None,
              data_type: str = None,
              order: str = None,
              print_url: bool = False,
              ) -> [BrokerItem]:
        params = []
        if start_ts:
            check_type(start_ts, int)
            params.append(f"start_ts={start_ts}")
        if end_ts:
            check_type(end_ts, int)
            params.append(f"end_ts={end_ts}")
        if collector:
            check_type(collector, str)
            params.append(f"collector={collector}")
        if project:
            check_type(project, str)
            params.append(f"project={project}")
        if data_type:
            check_type(data_type, str)
            params.append(f"data_type={data_type}")
        if order:
            check_type(order, str)
            params.append(f"order={order}")
        params.append(f"page_size={self.page_size}")

        api_url = f"{self.base_url}/search?" + "&".join(params)
        page = 1

        data_items = []
        if print_url:
            print(api_url)
        res = requests.get(api_url).json()["data"]
        while res:
            if res["count"] > 0:
                data_items.extend([BrokerItem(**i) for i in res["items"]])

                if res["count"] < res["page_size"]:
                    break

                page += 1
                query_url = f"{api_url}&page={page}"
                if print_url:
                    print(query_url)
                res = requests.get(query_url).json()["data"]
            else:
                break

        return data_items



from dataclasses import dataclass
from typing import Optional, List

import requests as requests


def check_type(value: any, ty: type) -> bool:
    try:
        ty(value)
        return True
    except ValueError:
        raise ValueError("invalid option input")


@dataclass
class RoasItem:
    prefix: str
    asn: int
    tal: str
    date_ranges: List[List[str]]


@dataclass
class RoasRes:
    limit: int
    count: int
    data: List[RoasItem]
    next_page_num: Optional[int]
    next_page: Optional[str]
    error: Optional[str]


class Roas:

    def __init__(self, api_url: str = "https://api.roas.bgpkit.com"):
        self.base_url = api_url.strip()

    def query(self,
              prefix: str = None,
              asn: int = None,
              tal: str = None,
              date: str = None,
              max_len: int = None,
              debug: bool = False,
              ) -> [RoasItem]:

        if not (prefix or asn or tal or date or max_len):
            print("ERROR: must specify at least one query parameter: prefix, asn, tal, date, max_len")
            return []

        params = []
        if prefix:
            check_type(prefix, str)
            params.append(f"prefix={prefix}")
        if asn:
            check_type(asn, int)
            params.append(f"asn={asn}")
        if tal:
            check_type(tal, str)
            params.append(f"tal={tal}")
        if date:
            check_type(date, str)
            params.append(f"date={date}")
        if max_len:
            check_type(max_len, int)
            params.append(f"max_len={max_len}")

        api_url = f"{self.base_url}/lookup?" + "&".join(params)
        data_items = []
        if debug:
            print(api_url)
        res = RoasRes(**requests.get(api_url).json())
        while res.data:
            data_items.extend(res.data)
            if res.next_page:
                res = RoasRes(**requests.get(res.next_page).json())
            else:
                break

        return data_items



import setuptools

setuptools.setup(
    name='pybgpkit',
    version='0.0.2',
    description='BGPKIT tools Python bindings',
    url='https://github.com/bgpkit/pybgpkit',
    author='Mingwei Zhang',
    author_email='mingwei@bgpkit.com',
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
        # available on pip
        'pybgpkit-parser==0.0.1',
        'requests',
    ],
    entry_points={'console_scripts': [
    ]}
)
